@@ -5,8 +5,7 @@ The JIP job lister
 Usage:
     jip-jobs [-s <state>...] [-o <out>...] [-e]
              [--show-archived] [-j <id>...] [-J <cid>...]
-             [-N] [-q <queue>]
-    jip-jobs [-I <inputs>...] [-O <outputs>...] [-o <out>...] [-e]
+             [-N] [-q <queue>] [-I <inputs>...] [-O <outputs>...]
     jip-jobs [--help|-h]
 
 Options:
@@ -214,7 +213,10 @@ def SWITCH_PIPELINE_COLOR():
 def _pipeline_name(j):
     if LAST and LAST.pipeline != j.pipeline:
         SWITCH_PIPELINE_COLOR()
-    return colorize(j.pipeline, PIPELINE_COLOR)
+    if j.pipeline:
+        return colorize(j.pipeline, PIPELINE_COLOR)
+    else:
+        return ""
 
 
 JOB_HEADER = [
