@@ -431,6 +431,10 @@ def delete(job, clean_logs=False, cluster=None):
     does **NOT** delete any dependencies, it operates *ONLY* on the given
     job instance.
 
+    You can use :py:func:`jip.jobs.get_subgraph` to get a full subgraph of a
+    job, or :py:func:`jip.jobs.get_group_jobs` to create a list of all jobs
+    that are related due to grouping or piping.
+
     :param job: the job to be deleted
     :type job: `jip.db.Job`
     :param clean: if True, the job log files will be deleted
@@ -614,7 +618,17 @@ def submit_job(job, clean=False, force=False, save=True,
         session = db.commit_session(session)
         session.close()
 
-    # submit the job to the cluster
+    # Issue #12
+    # we have to make sure that log file folders exist
+    # otherwise job submission might succeed but nothing
+    # will be executed and the job failes silently without log files
+    for log_file in (job.stdout, job.stderr):
+        if not log_file:
+            continue
+        parent = os.path.dirname(log_file)
+        if not os.path.exists(parent):
+            os.makedirs(parent)
+    # submit the job
     cluster.submit(job)
     all_jobs = [job]
 
