@@ -1230,10 +1230,13 @@ class Pipeline(object):
                                   "%s [%s]", link, new_edge)
                 # get the new value
                 o = opts[j]
-                cloned_node.set(option.name, o, set_dep=False)
+                allow_stream = False
+                cloned_node.set(option.name, o, set_dep=False,
+                                allow_stream=allow_stream)
                 #cloned_node._tool.options[option.name]._index = i
+                ost = str(o) if not isinstance(o, Option) else o._value
                 log.debug("Fanout | apply value %s: %s=%s", cloned_node,
-                          option.name, o)
+                          option.name, ost)
                 ooo = cloned_node._tool.options[option.name]
                 ooo.dependency = option.dependency
                 ooo._index = i
@@ -1689,7 +1692,7 @@ class Node(object):
             def_in = other._tool.options.get_default_input()
             log.debug("op node > :: %s->%s [%s<-%s]",
                       self, other, dout.name, def_in.name)
-            other.set(def_in.name, dout)
+            other.set(def_in.name, dout, allow_stream=False)
             return other
 
         # if the right hand side is an option, set
