@@ -35,6 +35,7 @@ from os.path import exists, basename, dirname, abspath
 import os
 import sys
 import types
+import shutil
 
 import jip.templates
 from jip.options import Options, TYPE_OUTPUT, TYPE_INPUT, Option
@@ -858,15 +859,6 @@ class PythonBlockUtils(object):
         :rtype: :class:`jip.pipelines.Node`
         """
         bash_node = self.pipeline.run('bash', cmd=command, **kwargs)
-        ctx = {}
-        self._update_context(ctx, kwargs, bash_node)
-        # add options and make sure
-        # the bash tool options take precedence
-        ctx['input'] = bash_node._tool.options['input']
-        ctx['output'] = bash_node._tool.options['output']
-        ctx['outfile'] = bash_node._tool.options['outfile']
-        cmd = render_template(command, **ctx)
-        bash_node.cmd = cmd
         return bash_node
 
     def _update_global_env(self, env):
@@ -1293,7 +1285,10 @@ class Tool(object):
         for outfile in outfiles:
             if exists(outfile):
                 log.warning("Tool cleanup! Removing: %s", outfile)
-                remove(outfile)
+                if os.path.isfile(outfile):
+                    remove(outfile)
+                elif os.path.isdir(outfile):
+                    shutil.rmtree(outfile)
 
     def get_output_files(self, sticky=True):
         """Yields a list of all output files for the options
